/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file DereferenceLess.h
 * Header file for CDereferenceLess class that implements comparison operator
 * for pointers or for pointer-like types.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////

#pragma once


/**
*
* Implements comparison operator for pointers or for pointer-like types.  Will
* work w/ NULLs.
* 
* Useful for containers that employ comparisons in the course of container
* management, .e.g., map or set.  The default STL comparison function that's
* used is less<T>, where T is the contained type.  This usually isn't what you
* want when T is a pointer.  You'd usually sort by object value rather than by
* its position in memory.
* 
* Examples:
* 	set<CMyObject*, CDereferenceLess> m_thingies;
* 	map<char*, int, CDereferenceLess> m_vecpcStuff;
*/

struct CDereferenceLess
{
	template<typename PtrType>
	bool operator()( PtrType pT1,			// parameters are pased by
		             PtrType pT2 ) const	// value, because we expect them
	{										// to be (or to act like) pointers
		// Defined such that a NULL pointer is less than any value pointed to
		if( pT1 == pT2 )
		{
			return false; // same means NOT less than
		}
		if( pT1 != 0 )
		{
			if( pT2 != 0 )
			{
				return *pT1 < *pT2;
			}
			return false; // pT2 == 0, pT1 not
		}
		return true; // pT1 == 0, pT2 not
	}
};
