#ifndef _UPHIL_H_
#define _UPHIL_H_

#pragma once

// A place to put a bunch of reusable uphil functions:
// 
// - loading firmware
// - set RS-232, DS101, or DS102
// - set parameters for each of those modes
//   baud, bit count, etc.
// - maybe listing compatible devices & what states they're in
// - what else?
// 
// It's currently designed to be built as an archive (.a), and
// depends on libusb types for some things.
//
// We haven't gotten around to dumping much in here yet.

struct libusb_context;
struct libusb_device;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Invokes visitor for evey attached Uphil device. Iteration continues
 * as long as visitor returns 0.
 * @param context The libusb_context object you wish to use, or NULL to use
 * the default context.
 * @param visitor Your callback function.
 * @param p Optional data to be passed to visitor.
 * @return The most recent return value from visitor, or 0 if there were
 * no devices attached.
 */
int uphil_visit(struct libusb_context *context, int (*visitor)(void *p,
    struct libusb_device *device), void *p);

/**
 * Invokes visitor for evey attached unprogrammed Uphil device. Iteration
 * continues as long as visitor returns 0.
 * @param context The libusb_context object you wish to use, or NULL to use
 * the default context.
 * @param visitor Your callback function.
 * @param p Optional data to be passed to visitor.
 * @return The most recent return value from visitor, or 0 if there were
 * no devices attached.
 */
int uphil_visit_unloaded(struct libusb_context *context, int (*visitor)(
    void *p, struct libusb_device *device), void *p);

/**
 * Finds and refs (libusb_ref_device) the Uphil device at the given bus and
 * address. On Linux the device shows up at /dev/bus/usb/<bus>/<address>,
 * though file system paths aren't (directly) used by this function.
 * Caller is responsible for unrefing (libusb_unref_device) the returned
 * object when done with it.
 * @param context The libusb_context object you wish to use, or NULL to use
 * the default context.
 * @return A valid libusb_device object or NULL on failure.
 */
struct libusb_device *uphil_ref(struct libusb_context *context, int bus, int
    address);

/**
 * Uploads firmware into the UPhil device at the given bus and address.
 * On Linux the device shows up at /dev/bus/usb/<bus>/<address>,
 * though file system paths aren't (directly) used by this function.
 * @return 0 on success.
 */
int uphil_load_firmware(int bus, int address);

/**
 * Uploads firmware into the libusb_device object. Hopefully it's a UPhil
 * device. On Linux the device shows up at /dev/bus/usb/<bus>/<address>,
 * though file system paths aren't (directly) used by this function.
 * @return 0 on success.
 */
int uphil_load_firmware2(struct libusb_device *device);

#ifdef __cplusplus
}
#endif

#endif // _UPHIL_H_
